<?php

namespace Drupal\silktide;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\silktide\event\SilktideEvent;
use GuzzleHttp\ClientInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class SilktideService implements EventSubscriberInterface
{

  const SILKTIDE_URL = 'https://api.silktide.com/cms/update';

  const VERSION = '1.0.0';

  /**
   * The http client.
   *
   * @var ClientInterface
   */
  protected $client;

  /**
   * The logger channel factory.
   *
   * @var LoggerChannelFactoryInterface
   */
  protected $loggerFactory;


  /**
   * Constructor.
   *
   * @param ClientInterface $client
   *   A http client.
   * @param LoggerChannelFactoryInterface $logger_factory
   *   A logger channel factory.
   */
  public function __construct(
    ClientInterface $client,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    $this->client = $client;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * @return array|mixed
   */
  public static function getSubscribedEvents()
  {
    $events[SilktideEvent::EVENT_NAME][] = ['notify'];
    return $events;
  }

  /**
   * @param SilktideEvent $event
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function notify(SilktideEvent $event)
  {
    $apiKey = \Drupal::config('silktide.settings')->get('apikey');
    if (32 !== strlen($apiKey)) {
      return;
    }
    try {
      $response = $this->client->request(
        'post',
        self::SILKTIDE_URL,
        [
          'headers' => [
            'User-Agent' => 'SilktideDrupal/' . self::VERSION . ' (compatible; Drupal/' . \Drupal::VERSION . ')'
          ],
          'body' => http_build_query(
            [
              'apiKey' => $apiKey,
              'urls' => [$event->getUrl()]
            ]
          )
        ]
      );
      $this->loggerFactory->get('silktide')->info(
        'Notified Silktide @message',
        ['@message' => $response->getBody()->getContents()]
      );
    } catch (\Throwable $e) {
      $this->loggerFactory->get('silktide')->error(
        'Could not send notification to Silktide @message',
        ['@message' => $e->getMessage()]
      );
    }
  }
}
